@@ -1,14 +1,13 @@
 <?php
 
 /**
- * addressbook.php
+ * functions/addressbook.php - Functions and classes for the addressbook system
  *
- * Copyright (c) 1999-2005 The SquirrelMail Project Team
- * Licensed under the GNU GPL. For full terms see the file COPYING.
+ * Functions require SM_PATH and support of forms.php functions
  *
- * Functions and classes for the addressbook system.
- *
- * @version $Id: addressbook.php,v 1.47.2.14 2005/06/15 21:11:57 kink Exp $
+ * @copyright &copy; 1999-2009 The SquirrelMail Project Team
+ * @license http://opensource.org/licenses/gpl-license.php GNU Public License
+ * @version $Id: addressbook.php 13549 2009-04-15 22:00:49Z jervfors $
  * @package squirrelmail
  * @subpackage addressbook
  */
@@ -33,8 +32,12 @@ global $addrbook_dsn, $addrbook_global_dsn;
 function addressbook_init($showerr = true, $onlylocal = false) {
     global $data_dir, $username, $color, $ldap_server, $address_book_global_filename;
     global $addrbook_dsn, $addrbook_table;
-    global $abook_global_file, $abook_global_file_writeable;
+    // Shared file based address book globals
+    global $abook_global_file, $abook_global_file_writeable, $abook_global_file_listing;
+    // Shared DB based address book globals
     global $addrbook_global_dsn, $addrbook_global_table, $addrbook_global_writeable, $addrbook_global_listing;
+    // Record size restriction in file based address books
+    global $abook_file_line_length;
 
     /* Create a new addressbook object */
     $abook = new AddressBook;
@@ -56,13 +59,14 @@ function addressbook_init($showerr = true, $onlylocal = false) {
                             'owner' => $username,
                             'table' => $addrbook_table));
         if (!$r && $showerr) {
-            $abook_init_error.=_("Error initializing addressbook database.") .' '. $abook->error;
+            $abook_init_error.=_("Error initializing address book database.") .' '. $abook->error;
         }
     } else {
         /* File */
         $filename = getHashedFile($username, $data_dir, "$username.abook");
         $r = $abook->add_backend('local_file', Array('filename' => $filename,
-                              'create'   => true));
+                                                     'line_length' => $abook_file_line_length,
+                                                     'create'   => true));
         if(!$r && $showerr) {
             $abook_init_error.=sprintf( _("Error opening file %s"), $filename );
         }
@@ -89,24 +93,15 @@ function addressbook_init($showerr = true, $onlylocal = false) {
         $r = $abook->add_backend('local_file',array('filename'=>$abook_global_filename,
                                                     'name' => _("Global address book"),
                                                     'detect_writeable' => false,
-                                                    'writeable'=> $abook_global_file_writeable));
+                                                    'line_length' => $abook_file_line_length,
+                                                    'writeable'=> $abook_global_file_writeable,
+                                                    'listing' => $abook_global_file_listing));
         if (!$r && $showerr) {
             if ($abook_init_error!='') $abook_init_error.="\n";
-            $abook_init_error.=_("Error initializing global addressbook.") . "\n" . $abook->error;
+            $abook_init_error.=_("Error initializing global address book.") . "\n" . $abook->error;
         }
     }
 
-
-    /*
-     * hook allows to include different address book backends.
-     * plugins should extract $abook and $r from arguments
-     * and use same add_backend commands as above functions.
-     * @since 1.5.1 and 1.4.5
-     */
-    $hookReturn = do_hook('abook_init', $abook, $r);
-    $abook = $hookReturn[1];
-    $r = $hookReturn[2];
-
     /* Load global addressbook from SQL if configured */
     if (isset($addrbook_global_dsn) && !empty($addrbook_global_dsn)) {
         /* Database configured */
@@ -122,10 +117,20 @@ function addressbook_init($showerr = true, $onlylocal = false) {
                                        'table' => $addrbook_global_table));
         if (!$r && $showerr) {
             if ($abook_init_error!='') $abook_init_error.="\n";
-            $abook_init_error.=_("Error initializing global addressbook.") . "\n" . $abook->error;
+            $abook_init_error.=_("Error initializing global address book.") . "\n" . $abook->error;
     }
     }
 
+    /*
+     * hook allows to include different address book backends.
+     * plugins should extract $abook and $r from arguments
+     * and use same add_backend commands as above functions.
+     * @since 1.5.1 and 1.4.5
+     */
+    $hookReturn = do_hook('abook_init', $abook, $r);
+    $abook = $hookReturn[1];
+    $r = $hookReturn[2];
+
     if (! $onlylocal) {
     /* Load configured LDAP servers (if PHP has LDAP support) */
     if (isset($ldap_server) && is_array($ldap_server) && function_exists('ldap_connect')) {
@@ -173,6 +178,104 @@ function addressbook_cmp($a,$b) {
 
 }
 
+/**
+ * Sort array by the key "name"
+ */
+function alistcmp($a,$b) {
+    $abook_sort_order=get_abook_sort();
+
+    switch ($abook_sort_order) {
+    case 0:
+    case 1:
+      $abook_sort='nickname';
+      break;
+    case 4:
+    case 5:
+      $abook_sort='email';
+      break;
+    case 6:
+    case 7:
+      $abook_sort='label';
+      break;
+    case 2:
+    case 3:
+    case 8:
+    default:
+      $abook_sort='name';
+    }
+
+    if ($a['backend'] > $b['backend']) {
+        return 1;
+    } else {
+        if ($a['backend'] < $b['backend']) {
+            return -1;
+        }
+    }
+
+    if( (($abook_sort_order+2) % 2) == 1) {
+      return (strtolower($a[$abook_sort]) < strtolower($b[$abook_sort])) ? 1 : -1;
+    } else {
+      return (strtolower($a[$abook_sort]) > strtolower($b[$abook_sort])) ? 1 : -1;
+    }
+}
+
+/**
+ * Address book sorting options
+ *
+ * returns address book sorting order
+ * @return integer book sorting options order
+ */
+function get_abook_sort() {
+    global $data_dir, $username;
+
+    /* get sorting order */
+    if(sqgetGlobalVar('abook_sort_order', $temp, SQ_GET)) {
+      $abook_sort_order = (int) $temp;
+
+      if ($abook_sort_order < 0 or $abook_sort_order > 8)
+        $abook_sort_order=8;
+
+      setPref($data_dir, $username, 'abook_sort_order', $abook_sort_order);
+    } else {
+      /* get previous sorting options. default to unsorted */
+      $abook_sort_order = getPref($data_dir, $username, 'abook_sort_order', 8);
+    }
+
+    return $abook_sort_order;
+}
+
+/**
+ * This function shows the address book sort button.
+ *
+ * @param integer $abook_sort_order current sort value
+ * @param string $alt_tag alt tag value (string visible to text only browsers)
+ * @param integer $Down sort value when list is sorted ascending
+ * @param integer $Up sort value when list is sorted descending
+ * @return string html code with sorting images and urls
+ * @since 1.5.1 and 1.4.6
+ */
+function show_abook_sort_button($abook_sort_order, $alt_tag, $Down, $Up ) {
+    global $form_url;
+
+     /* Figure out which image we want to use. */
+    if ($abook_sort_order != $Up && $abook_sort_order != $Down) {
+        $img = 'sort_none.png';
+        $which = $Up;
+    } elseif ($abook_sort_order == $Up) {
+        $img = 'up_pointer.png';
+        $which = $Down;
+    } else {
+        $img = 'down_pointer.png';
+        $which = 8;
+    }
+
+      /* Now that we have everything figured out, show the actual button. */
+    return ' <a href="' . $form_url .'?abook_sort_order=' . $which
+         . '"><img src="../images/' . $img
+         . '" border="0" width="12" height="10" alt="' . $alt_tag . '" title="'
+         . _("Click here to change the sorting of the address list") .'" /></a>';
+}
+
 
 /**
  * This is the main address book class that connect all the
@@ -187,6 +290,7 @@ class AddressBook {
     var $error       = '';
     var $localbackend = 0;
     var $localbackendname = '';
+    var $add_extra_field = false;
 
       // Constructor function.
     function AddressBook() {
@@ -246,15 +350,28 @@ class AddressBook {
      */
 
     function full_address($row) {
-        global $addrsrch_fullname, $data_dir, $username;
-        $prefix = getPref($data_dir, $username, 'addrsrch_fullname');
-        if (($prefix != "" || (isset($addrsrch_fullname) &&
-            $prefix == $addrsrch_fullname)) && $prefix != 'noprefix') {
-            $name = ($prefix == 'nickname' ? $row['nickname'] : $row['name']);
-            return $name . ' <' . trim($row['email']) . '>';
-        } else {
-            return trim($row['email']);
+        global $data_dir, $username;
+        $addrsrch_fullname = getPref($data_dir, $username, 'addrsrch_fullname', 'fullname');
+
+        // allow multiple addresses in one row (poor person's grouping - bah)
+        // (separate with commas)
+        //
+        $return = '';
+        $addresses = explode(',', $row['email']);
+        foreach ($addresses as $address) {
+
+            if (!empty($return)) $return .= ', ';
+
+            if ($addrsrch_fullname == 'fullname')
+                $return .= '"' . $row['name'] . '" <' . trim($address) . '>';
+            else if ($addrsrch_fullname == 'nickname')
+                $return .= '"' . $row['nickname'] . '" <' . trim($address) . '>';
+            else // "noprefix"
+                $return .= trim($address);
+
         }
+
+        return $return;
     }
 
     /*
@@ -313,19 +430,19 @@ class AddressBook {
 
 
     /*
-     *  Lookup an address by alias. Only possible in
-     *  local backends.
+     *  Lookup an address by the indicated field. Only
+     *  possible in local backends.
      */
-    function lookup($alias, $bnum = -1) {
+    function lookup($value, $bnum = -1, $field = SM_ABOOK_FIELD_NICKNAME) {
 
         $ret = array();
 
         if ($bnum > -1) {
-            $res = $this->backends[$bnum]->lookup($alias);
+            $res = $this->backends[$bnum]->lookup($value, $field);
             if (is_array($res)) {
                return $res;
             } else {
-               $this->error = $backend->error;
+               $this->error = $this->backends[$bnum]->error;
                return false;
             }
         }
@@ -334,13 +451,18 @@ class AddressBook {
         for ($i = 0 ; $i < sizeof($sel) ; $i++) {
             $backend = &$sel[$i];
             $backend->error = '';
-            $res = $backend->lookup($alias);
+            $res = $backend->lookup($value, $field);
+
+            // return an address if one is found
+            // (empty array means lookup concluded
+            // but no result found - in this case,
+            // proceed to next backend)
+            //
             if (is_array($res)) {
-               if(!empty($res))
-              return $res;
+                if (!empty($res)) return $res;
             } else {
-               $this->error = $backend->error;
-               return false;
+                $this->error = $backend->error;
+                return false;
             }
         }
 
@@ -404,7 +526,7 @@ class AddressBook {
 
         /* Check that specified backend accept new entries */
         if (!$this->backends[$bnum]->writeable) {
-            $this->error = _("Addressbook is read-only");
+            $this->error = _("Address book is read-only");
             return false;
         }
 
@@ -439,7 +561,7 @@ class AddressBook {
 
         /* Check that specified backend is writable */
         if (!$this->backends[$bnum]->writeable) {
-            $this->error = _("Addressbook is read-only");
+            $this->error = _("Address book is read-only");
             return false;
         }
 
@@ -492,7 +614,7 @@ class AddressBook {
 
         /* Check that specified backend is writable */
         if (!$this->backends[$bnum]->writeable) {
-            $this->error = _("Addressbook is read-only");;
+            $this->error = _("Address book is read-only");;
             return false;
         }
 
@@ -543,7 +665,7 @@ class addressbook_backend {
         return false;
     }
 
-    function lookup($alias) {
+    function lookup($value, $field) {
         $this->set_error('lookup not implemented');
         return false;
     }
@@ -570,21 +692,6 @@ class addressbook_backend {
 
 }
 
-/**
- * Sort array by the key "name"
- */
-function alistcmp($a,$b) {
-    if ($a['backend'] > $b['backend']) {
-        return 1;
-    } else {
-        if ($a['backend'] < $b['backend']) {
-            return -1;
-        }
-    }
-    return (strtolower($a['name']) > strtolower($b['name'])) ? 1 : -1;
-}
-
-
 /*
   PHP 5 requires that the class be made first, which seems rather
   logical, and should have been the way it was generated the first time.
@@ -607,4 +714,3 @@ if((isset($addrbook_dsn) && !empty($addrbook_dsn)) ||
  * @since 1.5.1 and 1.4.5
  */
 do_hook('abook_add_class');
-?>
