@@ -3,13 +3,12 @@
 /**
  * display_messages.php
  *
- * Copyright (c) 1999-2005 The SquirrelMail Project Team
- * Licensed under the GNU GPL. For full terms see the file COPYING.
- *
  * This contains all messages, including information, error, and just
  * about any other message you can think of.
  *
- * @version $Id: display_messages.php,v 1.57.2.16 2005/03/21 22:18:32 kink Exp $
+ * @copyright &copy; 1999-2009 The SquirrelMail Project Team
+ * @license http://opensource.org/licenses/gpl-license.php GNU Public License
+ * @version $Id: display_messages.php 13549 2009-04-15 22:00:49Z jervfors $
  * @package squirrelmail
  */
 
@@ -18,35 +17,22 @@
  */
 require_once(SM_PATH . 'functions/plugin.php');
 
-/**
- * Find out where squirrelmail lives and try to be smart about it.
- * The only problem would be when squirrelmail lives in directories
- * called "src", "functions", or "plugins", but people who do that need
- * to be beaten with a steel pipe anyway.
- *
- * @return string the base uri of squirrelmail installation.
- */
-function sqm_baseuri(){
-    global $base_uri, $PHP_SELF;
-    /**
-     * If it is in the session, just return it.
-     */
-    if (isset($base_uri)){
-        return $base_uri;
-    }
-    $dirs = array('|src/.*|', '|plugins/.*|', '|functions/.*|');
-    $repl = array('', '', '');
-    $base_uri = preg_replace($dirs, $repl, $PHP_SELF);
-    return $base_uri;
-}
-
 function error_message($message, $mailbox, $sort, $startMessage, $color) {
+
+    global $default_folder_prefix;
+
     $urlMailbox = urlencode($mailbox);
     $string = '<tr><td align="center">' . $message . '</td></tr>'.
               '<tr><td align="center">'.
-              '<a href="'.sqm_baseuri()."src/right_main.php?sort=$sort&amp;startMessage=$startMessage&amp;mailbox=$urlMailbox\">".
-              sprintf (_("Click here to return to %s"),
-                  imap_utf7_decode_local($mailbox)).
+              '<a href="'.sqm_baseuri()."src/right_main.php?sort=$sort&amp;startMessage=$startMessage&amp;mailbox=$urlMailbox\">";
+
+    if (!empty($default_folder_prefix)) {
+        if (strpos($mailbox, $default_folder_prefix) === 0)
+            $mailbox = substr($mailbox, strlen($default_folder_prefix));
+    }
+    
+    $string .= sprintf (_("Click here to return to %s"),
+                  htmlspecialchars(imap_utf7_decode_local($mailbox))).
               '</a></td></tr>';
     error_box($string, $color);
 }
@@ -57,15 +43,18 @@ function plain_error_message($message, $color) {
 
 function logout_error( $errString, $errTitle = '' ) {
     global $frame_top, $org_logo, $org_name, $org_logo_width, $org_logo_height,
-           $hide_sm_attributions, $version, $squirrelmail_language, $color;
+           $hide_sm_attributions, $version, $squirrelmail_language, 
+           $color, $theme, $theme_default;
+
+    include_once( SM_PATH . 'functions/page_header.php' );
 
     $base_uri = sqm_baseuri();
 
-    include_once( SM_PATH . 'functions/page_header.php' );
-    if ( !isset( $org_logo ) ) {
-        // Don't know yet why, but in some accesses $org_logo is not set.
-        include( SM_PATH . 'config/config.php' );
-    }
+    $logout_link = $base_uri . 'src/login.php';
+
+    list($junk, $errString, $errTitle, $logout_link) 
+        = do_hook('logout_error', $errString, $errTitle, $logout_link);
+
     /* Display width and height like good little people */
     $width_and_height = '';
     if (isset($org_logo_width) && is_numeric($org_logo_width) && $org_logo_width>0) {
@@ -79,6 +68,10 @@ function logout_error( $errString, $errTitle = '' ) {
         $frame_top = '_top';
     }
 
+    // load default theme if possible
+    if (!isset($color) && @file_exists($theme[$theme_default]['PATH']))
+        @include ($theme[$theme_default]['PATH']);
+
     if ( !isset( $color ) ) {
         $color = array();
         $color[0]  = '#dcdcdc';  /* light gray    TitleBar               */
@@ -89,8 +82,6 @@ function logout_error( $errString, $errTitle = '' ) {
         $color[8]  = '#000000';  /* black         Normal text            */
     }
 
-    list($junk, $errString, $errTitle) = do_hook('logout_error', $errString, $errTitle);
-
     if ( $errTitle == '' ) {
         $errTitle = $errString;
     }
@@ -117,13 +108,13 @@ function logout_error( $errString, $errTitle = '' ) {
          '<tr><td align="center">' . $errString . '</td></tr>'.
          '<tr><td bgcolor="'.$color[0].'" align="center">'.
          '<font color="'.$color[2].'"><b>'.
-         '<a href="'.$base_uri.'src/login.php" target="'.$frame_top.'">'.
+         '<a href="'.$logout_link.'" target="'.$frame_top.'">'.
          _("Go to the login page") . '</a></b></font></td></tr>'.
          '</table></td></tr></table></center></body></html>';
 }
 
 function error_box($string, $color) {
-    global $pageheader_sent;
+    global $pageheader_sent, $org_title;
 
     if ( !isset( $color ) ) {
         $color = array();
@@ -135,6 +126,9 @@ function error_box($string, $color) {
         $color[8]  = '#000000';  /* black         Normal text            */
         $color[9]  = '#ababab';  /* mid-gray      Darker version of #0   */
     }
+    if ( !isset( $org_title ) ) {
+        $org_title = "SquirrelMail";
+    }
 
     $err = _("ERROR");
 
@@ -147,7 +141,7 @@ function error_box($string, $color) {
     if(!isset($pageheader_sent) && !$pageheader_sent) {
         /* include this just to be sure */
         include_once( SM_PATH . 'functions/page_header.php' );
-        displayHtmlHeader('SquirrelMail: '.$err);
+        displayHtmlHeader($org_title.': '.$err);
         $pageheader_sent = TRUE;
         echo "<body text=\"$color[8]\" bgcolor=\"$color[4]\" link=\"$color[7]\" vlink=\"$color[7]\" alink=\"$color[7]\">\n\n";
     }
@@ -176,5 +170,3 @@ function error_option_save($message) {
 
     $optpage_save_error=array_merge($optpage_save_error,array($message));
 }
-// vim: et ts=4
-?>
