@@ -3,15 +3,15 @@
 /**
  * html.php
  *
- * Copyright (c) 1999-2006 The SquirrelMail Project Team
- * Licensed under the GNU GPL. For full terms see the file COPYING.
- *
  * The idea is to inlcude here some functions to make easier
  * the right to left implementation by "functionize" some
  * html outputs.
  *
- * @version $Id: html.php,v 1.14.2.7 2006/02/03 22:27:47 jervfors Exp $
+ * @copyright &copy; 1999-2009 The SquirrelMail Project Team
+ * @license http://opensource.org/licenses/gpl-license.php GNU Public License
+ * @version $Id: html.php 13549 2009-04-15 22:00:49Z jervfors $
  * @package squirrelmail
+ * @since 1.3.0
  */
 
 /**
@@ -89,64 +89,65 @@ function html_tag( $tag,                // Tag to output
         return( $ret );
     }
 
-    /* handy function to set url vars */
-    /* especially usefull when $url = $PHP_SELF */
-    function set_url_var($url, $var, $val=0, $link=true) {
-        $k = '';
-        $ret = '';
-        $pat_a = array (
-                       '/.+(\\&'.$var.')=(.*)\\&/AU',   /* in the middle */
-                       '/.+\\?('.$var.')=(.*\\&).+/AU', /* at front, more follow */
-                       '/.+(\\?'.$var.')=(.*)$/AU',     /* at front and only var */
-                       '/.+(\\&'.$var.')=(.*)$/AU'      /* at the end */
-                     );
-	preg_replace('/&amp;/','&',$url);
-        switch (true) {
-            case (preg_match($pat_a[0],$url,$regs)):
-                $k = $regs[1];
-                $v = $regs[2];
-                break;
-            case (preg_match($pat_a[1],$url,$regs)):
-                $k = $regs[1];
-                $v = $regs[2];
-                break;
-            case (preg_match($pat_a[2],$url,$regs)):
-                $k = $regs[1];
-                $v = $regs[2];
-                break;
-            case (preg_match($pat_a[3],$url,$regs)):
-                $k = $regs[1];
-                $v = $regs[2];
-                break;
-            default:
-                if ($val) {
-                    if (strpos($url,'?')) {
-                        $url .= "&$var=$val";
-                    } else {
-                        $url .= "?$var=$val";
-                    }
-                }
-                break;
-        }
+/**
+ * This function is used to add, modify or delete GET variables in a URL. 
+ * It is especially useful when $url = $PHP_SELF
+ *
+ * Set $val to NULL to remove $var from $url.
+ * To ensure compatibility with older versions, use $val='0' to set $var to 0. 
+ *
+ * @param string $url url that must be modified
+ * @param string $var GET variable name
+ * @param string $val variable value
+ * @param boolean $link controls sanitizing of ampersand in urls (since 1.3.2)
+ *
+ * @return string $url modified url
+ *
+ * @since 1.3.0
+ *
+ */
+function set_url_var($url, $var, $val=null, $link=true) {
+    $url = str_replace('&amp;','&',$url);
 
-        if ($k) {
-            if ($val) {
-                $rpl = "$k=$val";
-		if ($link) {
-		    $rpl = preg_replace('/&/','&amp;',$rpl);
-		}
-            } else {
-                $rpl = '';
-            }
-            if( substr($v,-1)=='&' ) {
-                $rpl .= '&';
-            }
-            $pat = "/$k=$v/";
-            $url = preg_replace($pat,$rpl,$url);
+    if (strpos($url, '?') === false) {
+        $url .= '?';
+    }   
+
+    list($uri, $params) = explode('?', $url, 2);
+        
+    $newpar = array(); 
+    $params = explode('&', $params);
+   
+    foreach ($params as $p) {
+        if (trim($p)) {
+            $p = explode('=', $p);
+            $newpar[$p[0]] = (isset($p[1]) ? $p[1] : '');
         }
-        return $url;
     }
 
+    if (is_null($val)) {
+        unset($newpar[$var]);
+    } else {
+        $newpar[$var] = $val;
+    }
+
+    if (!count($newpar)) {
+        return $uri;
+    }
+   
+    $url = $uri . '?';
+    foreach ($newpar as $name => $value) {
+        $url .= "$name=$value&";
+    }
+     
+    $url = substr($url, 0, -1);
+    if ($link) {
+        $url = str_replace('&','&amp;',$url);
+    }
+    
+    return $url;
+}
+
     /* Temporary test function to proces template vars with formatting.
      * I use it for viewing the message_header (view_header.php) with
      * a sort of template.
@@ -182,4 +183,3 @@ function html_tag( $tag,                // Tag to output
             echo $format_ar[$frm_last];
         }
     }
-?>
\ No newline at end of file
