@@ -3,57 +3,20 @@
 /**
  * imap_mailbox.php
  *
- * Copyright (c) 1999-2004 The SquirrelMail Project Team
- * Licensed under the GNU GPL. For full terms see the file COPYING.
+ * This implements all functions that manipulate mailboxes
  *
- * This impliments all functions that manipulate mailboxes
- *
- * $Id: imap_mailbox.php,v 1.172.2.10 2004/05/01 14:32:01 kink Exp $
+ * @copyright &copy; 1999-2009 The SquirrelMail Project Team
+ * @license http://opensource.org/licenses/gpl-license.php GNU Public License
+ * @version $Id: imap_mailbox.php 13549 2009-04-15 22:00:49Z jervfors $
+ * @package squirrelmail
+ * @subpackage imap
  */
+
+/** UTF7 support */
 require_once(SM_PATH . 'functions/imap_utf7_local.php');
 
 global $boxesnew;
 
-class mailboxes {
-    var $mailboxname_full = '', $mailboxname_sub= '', $is_noselect = false, 
-        $is_special = false, $is_root = false, $is_inbox = false, $is_sent = false,
-        $is_trash = false, $is_draft = false,  $mbxs = array(), 
-        $unseen = false, $total = false;
-
-    function addMbx($mbx, $delimiter, $start, $specialfirst) {
-        $ary = explode($delimiter, $mbx->mailboxname_full);
-        $mbx_parent = &$this;
-        for ($i = $start, $c = count($ary)-1; $i < $c; $i++) {
-            $mbx_childs = &$mbx_parent->mbxs;
-            $found = false;
-            if ($mbx_childs) {
-                foreach ($mbx_childs as $key => $parent) {
-                    if ($parent->mailboxname_sub == $ary[$i]) {
-                        $mbx_parent = &$mbx_parent->mbxs[$key];
-                        $found = true;
-                    }
-                }
-            }
-            if (!$found) {
-                $no_select_mbx = new mailboxes();
-                if (isset($mbx_parent->mailboxname_full) && $mbx_parent->mailboxname_full != '') {
-                    $no_select_mbx->mailboxname_full = $mbx_parent->mailboxname_full.$delimiter.$ary[$i];
-                } else {
-                    $no_select_mbx->mailboxname_full = $ary[$i];
-                }
-                $no_select_mbx->mailboxname_sub = $ary[$i];
-                $no_select_mbx->is_noselect = true;
-                $mbx_parent->mbxs[] = $no_select_mbx;
-                $i--;
-            }
-        }
-        $mbx_parent->mbxs[] = $mbx;
-        if ($mbx->is_special && $specialfirst) {
-            usort($mbx_parent->mbxs, 'sortSpecialMbx');
-        }
-    }
-}
-
 function sortSpecialMbx($a, $b) {
     if ($a->is_inbox) {
         $acmp = '0'. $a->mailboxname_full;
@@ -74,7 +37,7 @@ function sortSpecialMbx($a, $b) {
 }
 
 function find_mailbox_name ($mailbox) {
-    if (preg_match('/\*.+\"([^\r\n\"]*)\"[\s\r\n]*$/', $mailbox, $regs)) 
+    if (preg_match('/\*.+\"([^\r\n\"]*)\"[\s\r\n]*$/', $mailbox, $regs))
         return $regs[1];
     if (ereg(" *\"([^\r\n\"]*)\"[ \r\n]*$", $mailbox, $regs))
         return $regs[1];
@@ -82,8 +45,11 @@ function find_mailbox_name ($mailbox) {
     return $regs[1];
 }
 
+/**
+ * @return bool whether this is a Noselect mailbox.
+ */
 function check_is_noselect ($lsub_line) {
-    return preg_match("/^\* LSUB \([^\)]*\\Noselect[^\)]*\)/i", $lsub_line);
+    return preg_match("/^\* (LSUB|LIST) \([^\)]*\\\\Noselect[^\)]*\)/i", $lsub_line);
 }
 
 /**
@@ -105,20 +71,20 @@ function readMailboxParent($haystack, $needle) {
     return( $ret );
 }
 
-/** 
+/**
  * Check if $subbox is below the specified $parentbox
  */
 function isBoxBelow( $subbox, $parentbox ) {
     global $delimiter;
-    /* 
-     * Eliminate the obvious mismatch, where the 
+    /*
+     * Eliminate the obvious mismatch, where the
      * subfolder path is shorter than that of the potential parent
      */
     if ( strlen($subbox) < strlen($parentbox) ) {
       return false;
     }
     /* check for delimiter */
-        if (!substr($parentbox,-1) == $delimiter) {
+        if (substr($parentbox,-1) != $delimiter) {
             $parentbox.=$delimiter;
         }
         if (substr($subbox,0,strlen($parentbox)) == $parentbox) {
@@ -128,39 +94,79 @@ function isBoxBelow( $subbox, $parentbox ) {
         }
 }
 
-/* Defines special mailboxes */
-function isSpecialMailbox( $box ) {
-    global $trash_folder, $sent_folder, $draft_folder,
-           $move_to_trash, $move_to_sent, $save_as_draft;
-
+/**
+ * Defines special mailboxes: given a mailbox name, it checks if this is a
+ * "special" one: INBOX, Trash, Sent or Draft.
+ *
+ * Since 1.2.5 function includes special_mailbox hook.<br>
+ * Since 1.4.3 hook supports more than one plugin.
+ * @param string $box mailbox name
+ * @param boolean $include_subs (since 1.5.2 and 1.4.9) if true, subfolders of 
+ *  system folders are special. if false, subfolders are not special mailboxes 
+ *  unless they are tagged as special in 'special_mailbox' hook.
+ * @return boolean
+ * @since 1.2.3
+ */
+function isSpecialMailbox($box,$include_subs=true) {
     $ret = ( (strtolower($box) == 'inbox') ||
-             isTrashMailbox($box) || isSentMailbox($box) || isDraftMailbox($box) );
+             isTrashMailbox($box,$include_subs) || 
+             isSentMailbox($box,$include_subs) || 
+             isDraftMailbox($box,$include_subs) );
 
     if ( !$ret ) {
-        $ret = boolean_hook_function( 'special_mailbox', $box, 1 );
+        $ret = boolean_hook_function('special_mailbox',$box,1);
     }
     return $ret;
 }
 
-function isTrashMailbox ($box) {
+/**
+ * Detects if mailbox is a Trash folder or subfolder of Trash
+ * @param string $box mailbox name
+ * @param boolean $include_subs (since 1.5.2 and 1.4.9) if true, subfolders of 
+ *  system folders are special. if false, subfolders are not special mailboxes.
+ * @return bool whether this is a Trash folder
+ * @since 1.4.0
+ */
+function isTrashMailbox ($box,$include_subs=true) {
     global $trash_folder, $move_to_trash;
     return $move_to_trash && $trash_folder &&
-           ( $box == $trash_folder || isBoxBelow($box, $trash_folder) );
+           ( $box == $trash_folder || 
+             ($include_subs && isBoxBelow($box, $trash_folder)) );
 }
 
-function isSentMailbox($box) {
+/**
+ * Detects if mailbox is a Sent folder or subfolder of Sent
+ * @param string $box mailbox name
+ * @param boolean $include_subs (since 1.5.2 and 1.4.9) if true, subfolders of 
+ *  system folders are special. if false, subfolders are not special mailboxes.
+ * @return bool whether this is a Sent folder
+ * @since 1.4.0
+ */
+function isSentMailbox($box,$include_subs=true) {
    global $sent_folder, $move_to_sent;
    return $move_to_sent && $sent_folder &&
-          ( $box == $sent_folder || isBoxBelow($box, $sent_folder) );
+          ( $box == $sent_folder || 
+            ($include_subs && isBoxBelow($box, $sent_folder)) );
 }
 
-function isDraftMailbox($box) {
+/**
+ * Detects if mailbox is a Drafts folder or subfolder of Drafts
+ * @param string $box mailbox name
+ * @param boolean $include_subs (since 1.5.2 and 1.4.9) if true, subfolders of 
+ *  system folders are special. if false, subfolders are not special mailboxes.
+ * @return bool whether this is a Draft folder
+ * @since 1.4.0
+ */
+function isDraftMailbox($box,$include_subs=true) {
    global $draft_folder, $save_as_draft;
    return $save_as_draft &&
-          ( $box == $draft_folder || isBoxBelow($box, $draft_folder) );
+          ( $box == $draft_folder || 
+            ($include_subs && isBoxBelow($box, $draft_folder)) );
 }
 
-/* Expunges a mailbox */
+/**
+ * Expunges a mailbox, ie. delete all contents.
+ */
 function sqimap_mailbox_expunge ($imap_stream, $mailbox, $handle_errors = true, $id='') {
     global $uid_support;
     if ($id) {
@@ -186,7 +192,9 @@ function sqimap_mailbox_expunge ($imap_stream, $mailbox, $handle_errors = true,
     return $cnt;
 }
 
-/* Checks whether or not the specified mailbox exists */
+/**
+ * Checks whether or not the specified mailbox exists
+ */
 function sqimap_mailbox_exists ($imap_stream, $mailbox) {
     if (!isset($mailbox) || empty($mailbox)) {
         return false;
@@ -196,14 +204,36 @@ function sqimap_mailbox_exists ($imap_stream, $mailbox) {
     return isset($mbx[0]);
 }
 
-/* Selects a mailbox */
+/**
+ * Selects a mailbox
+ */
 function sqimap_mailbox_select ($imap_stream, $mailbox) {
     global $auto_expunge;
 
-    if ($mailbox == 'None') {
+    if (empty($mailbox)) {
         return;
     }
 
+    /**
+     * Default UW IMAP server configuration allows to access other files
+     * on server. $imap_server_type is not checked because interface can
+     * be used with 'other' or any other server type setting. $mailbox
+     * variable can be modified in any script that uses variable from GET 
+     * or POST. This code blocks all standard SquirrelMail IMAP API requests 
+     * that use mailbox with full path (/etc/passwd) or with ../ characters 
+     * in path (../../etc/passwd)
+     */
+    if (strstr($mailbox, '../') || substr($mailbox, 0, 1) == '/') {
+        global $color;
+        include_once(SM_PATH . 'functions/display_messages.php');
+        error_box(sprintf(_("Invalid mailbox name: %s"),htmlspecialchars($mailbox)),$color);
+        sqimap_logout($imap_stream);
+        die('</body></html>');
+    }
+
+    // cleanup $mailbox in order to prevent IMAP injection attacks
+    $mailbox = str_replace(array("\r","\n"), array("",""),$mailbox);
+
     $read = sqimap_run_command($imap_stream, "SELECT \"$mailbox\"",
                                true, $response, $message);
     $result = array();
@@ -232,19 +262,25 @@ function sqimap_mailbox_select ($imap_stream, $mailbox) {
     return $result;
 }
 
-/* Creates a folder */
+/**
+ * Creates a folder.
+ */
 function sqimap_mailbox_create ($imap_stream, $mailbox, $type) {
     global $delimiter;
     if (strtolower($type) == 'noselect') {
-        $mailbox .= $delimiter;
+        $create_mailbox = $mailbox . $delimiter;
+    } else {
+        $create_mailbox = $mailbox;
     }
 
-    $read_ary = sqimap_run_command($imap_stream, "CREATE \"$mailbox\"",
+    $read_ary = sqimap_run_command($imap_stream, "CREATE \"$create_mailbox\"",
                                    true, $response, $message);
     sqimap_subscribe ($imap_stream, $mailbox);
 }
 
-/* Subscribes to an existing folder */
+/**
+ * Subscribes to an existing folder.
+ */
 function sqimap_subscribe ($imap_stream, $mailbox) {
     $read_ary = sqimap_run_command($imap_stream, "SUBSCRIBE \"$mailbox\"",
                                    true, $response, $message);
@@ -264,18 +300,23 @@ function sqimap_unsubscribe ($imap_stream, $mailbox) {
 function sqimap_mailbox_delete ($imap_stream, $mailbox) {
     global $data_dir, $username;
     sqimap_unsubscribe ($imap_stream, $mailbox);
-    $read_ary = sqimap_run_command($imap_stream, "DELETE \"$mailbox\"",
-                                   true, $response, $message);
-    if ($response !== 'OK') {
-        // subscribe again
-        sqimap_subscribe ($imap_stream, $mailbox);
-    } else {
-        do_hook_function('rename_or_delete_folder', $args = array($mailbox, 'delete', ''));
-        removePref($data_dir, $username, "thread_$mailbox");
+    if (sqimap_mailbox_exists($imap_stream, $mailbox)) {
+        $read_ary = sqimap_run_command($imap_stream, "DELETE \"$mailbox\"",
+                                       true, $response, $message);
+        if ($response !== 'OK') {
+            // subscribe again
+            sqimap_subscribe ($imap_stream, $mailbox);
+        } else {
+            do_hook_function('rename_or_delete_folder', $args = array($mailbox, 'delete', ''));
+            removePref($data_dir, $username, "thread_$mailbox");
+            removePref($data_dir, $username, "collapse_folder_$mailbox");
+        }
     }
 }
 
-/* Determines if the user is subscribed to the folder or not */
+/**
+ * Determines if the user is subscribed to the folder or not
+ */
 function sqimap_mailbox_is_subscribed($imap_stream, $folder) {
     $boxesall = sqimap_mailbox_list ($imap_stream);
     foreach ($boxesall as $ref) {
@@ -286,7 +327,9 @@ function sqimap_mailbox_is_subscribed($imap_stream, $folder) {
     return false;
 }
 
-/* Renames a mailbox */
+/**
+ * Renames a mailbox.
+ */
 function sqimap_mailbox_rename( $imap_stream, $old_name, $new_name ) {
     if ( $old_name != $new_name ) {
         global $delimiter, $imap_server_type, $data_dir, $username;
@@ -298,14 +341,17 @@ function sqimap_mailbox_rename( $imap_stream, $old_name, $new_name ) {
             $postfix = '';
         }
 
-        $boxesall = sqimap_mailbox_list($imap_stream);
+        $boxesall = sqimap_mailbox_list_all($imap_stream);
         $cmd = 'RENAME "' . $old_name . '" "' . $new_name . '"';
         $data = sqimap_run_command($imap_stream, $cmd, true, $response, $message);
         sqimap_unsubscribe($imap_stream, $old_name.$postfix);
-        $oldpref = getPref($data_dir, $username, 'thread_'.$old_name.$postfix);
+        $oldpref_thread = getPref($data_dir, $username, 'thread_'.$old_name.$postfix);
+        $oldpref_collapse = getPref($data_dir, $username, 'collapse_folder_'.$old_name.$postfix);
         removePref($data_dir, $username, 'thread_'.$old_name.$postfix);
+        removePref($data_dir, $username, 'collapse_folder_'.$old_name.$postfix);
         sqimap_subscribe($imap_stream, $new_name.$postfix);
-        setPref($data_dir, $username, 'thread_'.$new_name.$postfix, $oldpref);
+        setPref($data_dir, $username, 'thread_'.$new_name.$postfix, $oldpref_thread);
+        setPref($data_dir, $username, 'collapse_folder_'.$new_name.$postfix, $oldpref_collapse);
         do_hook_function('rename_or_delete_folder',$args = array($old_name, 'rename', $new_name));
         $l = strlen( $old_name ) + 1;
         $p = 'unformatted';
@@ -313,16 +359,25 @@ function sqimap_mailbox_rename( $imap_stream, $old_name, $new_name ) {
         foreach ($boxesall as $box) {
             if (substr($box[$p], 0, $l) == $old_name . $delimiter) {
                 $new_sub = $new_name . $delimiter . substr($box[$p], $l);
+                /* With Cyrus IMAPd >= 2.0 rename is recursive, so don't check for errors here */
                 if ($imap_server_type == 'cyrus') {
                     $cmd = 'RENAME "' . $box[$p] . '" "' . $new_sub . '"';
-                    $data = sqimap_run_command($imap_stream, $cmd, true,
+                    $data = sqimap_run_command($imap_stream, $cmd, false,
                                                $response, $message);
                 }
-                sqimap_unsubscribe($imap_stream, $box[$p]);
-                $oldpref = getPref($data_dir, $username, 'thread_'.$box[$p]);
+                $was_subscribed = sqimap_mailbox_is_subscribed($imap_stream, $box[$p]);
+                if ( $was_subscribed ) {
+                    sqimap_unsubscribe($imap_stream, $box[$p]);
+                }
+                $oldpref_thread = getPref($data_dir, $username, 'thread_'.$box[$p]);
+                $oldpref_collapse = getPref($data_dir, $username, 'collapse_folder_'.$box[$p]);
                 removePref($data_dir, $username, 'thread_'.$box[$p]);
-                sqimap_subscribe($imap_stream, $new_sub);
-                setPref($data_dir, $username, 'thread_'.$new_sub, $oldpref);
+                removePref($data_dir, $username, 'collapse_folder_'.$box[$p]);
+                if ( $was_subscribed ) {
+                    sqimap_subscribe($imap_stream, $new_sub);
+                }
+                setPref($data_dir, $username, 'thread_'.$new_sub, $oldpref_thread);
+                setPref($data_dir, $username, 'collapse_folder_'.$new_sub, $oldpref_collapse);
                 do_hook_function('rename_or_delete_folder',
                                  $args = array($box[$p], 'rename', $new_sub));
             }
@@ -330,10 +385,10 @@ function sqimap_mailbox_rename( $imap_stream, $old_name, $new_name ) {
     }
 }
 
-/*
- * Formats a mailbox into 4 parts for the $boxesall array
+/**
+ * Formats a mailbox into parts for the $boxesall array
  *
- * The four parts are:
+ * The parts are:
  *
  *     raw            - Raw LIST/LSUB response from the IMAP server
  *     formatted      - nicely formatted folder name
@@ -354,7 +409,7 @@ function sqimap_mailbox_parse ($line, $line_lsub) {
         }
 
         /* Count number of delimiters ($delimiter) in folder name */
-        $mailbox  = trim($line_lsub[$g]);
+        $mailbox  = $line_lsub[$g];
         $dm_count = substr_count($mailbox, $delimiter);
         if (substr($mailbox, -1) == $delimiter) {
             /* If name ends in delimiter, decrement count by one */
@@ -402,7 +457,7 @@ function sqimap_mailbox_parse ($line, $line_lsub) {
     return $boxesall;
 }
 
-/*
+/**
  * Sorting function used to sort mailbox names.
  *     + Original patch from dave_michmerhuizen@yahoo.com
  *     + Allows case insensitivity when sorting folders
@@ -418,10 +473,10 @@ function user_strcasecmp($a, $b) {
     return  strnatcasecmp($a, $b);
 }
 
-/*
+/**
  * Returns list of options (to be echoed into select statement
  * based on available mailboxes and separators
- * Caller should surround options with <SELECT..> </SELECT> and
+ * Caller should surround options with <select ...> </select> and
  * any formatting.
  *   $imap_stream - $imapConnection to query for mailboxes
  *   $show_selected - array containing list of mailboxes to pre-select (0 if none)
@@ -434,15 +489,14 @@ function user_strcasecmp($a, $b) {
  *           NULL to avoid flag check entirely.
  *   $use_long_format - override folder display preference and always show full folder name.
  */
-function sqimap_mailbox_option_list($imap_stream, $show_selected = 0, $folder_skip = 0, $boxes = 0, 
+function sqimap_mailbox_option_list($imap_stream, $show_selected = 0, $folder_skip = 0, $boxes = 0,
                                     $flag = 'noselect', $use_long_format = false ) {
     global $username, $data_dir;
     $mbox_options = '';
-
     if ( $use_long_format ) {
         $shorten_box_names = 0;
     } else {
-        $shorten_box_names = getPref($data_dir, $username, 'mailbox_select_style', SMPREF_OFF);
+        $shorten_box_names = getPref($data_dir, $username, 'mailbox_select_style', 1);
     }
 
     if ($boxes == 0) {
@@ -456,42 +510,61 @@ function sqimap_mailbox_option_list($imap_stream, $show_selected = 0, $folder_sk
             if ($folder_skip != 0 && in_array($box, $folder_skip) ) {
                 continue;
             }
-            $lowerbox = strtolower($box); 
-	    // mailboxes are casesensitive => inbox.sent != inbox.Sent
-	    // nevermind, to many dependencies this should be fixed!
-	    
+            $lowerbox = strtolower($box);
+            // mailboxes are casesensitive => inbox.sent != inbox.Sent
+            // nevermind, to many dependencies this should be fixed!
+
             if (strtolower($box) == 'inbox') { // inbox is special and not casesensitive
                 $box2 = _("INBOX");
-            } else { 
-	        switch ($shorten_box_names)
-		{
-		  case 2:   /* delimited, style = 2 */
+            } else {
+                switch ($shorten_box_names)
+                {
+                  case 2:   /* delimited, style = 2 */
                     $box2 = str_replace('&nbsp;&nbsp;', '.&nbsp;', $boxes_part['formatted']);
-		    break;
+                    break;
                   case 1:   /* indent, style = 1 */
                     $box2 = $boxes_part['formatted'];
-		    break;
+                    break;
                   default:  /* default, long names, style = 0 */
-                    $box2 = str_replace(' ', '&nbsp;', imap_utf7_decode_local($boxes_part['unformatted-disp']));
-		    break;
-		}
+                    $box2 = str_replace(' ', '&nbsp;', htmlspecialchars(imap_utf7_decode_local($boxes_part['unformatted-disp'])));
+                    break;
+                }
             }
             $box2 = str_replace(array('<','>'), array('&lt;','&gt;') , $box2);
 
             if ($show_selected != 0 && in_array($lowerbox, $show_selected) ) {
-                $mbox_options .= '<OPTION VALUE="'.$box.'" SELECTED>'.$box2.'</OPTION>' . "\n";
+                $mbox_options .= '<option value="' . htmlspecialchars($box) .'" selected="selected">'.$box2.'</option>' . "\n";
             } else {
-                $mbox_options .= '<OPTION VALUE="'.$box.'">'.$box2.'</OPTION>' . "\n";
+                $mbox_options .= '<option value="' . htmlspecialchars($box) .'">'.$box2.'</option>' . "\n";
             }
         }
     }
     return $mbox_options;
 }
 
-/*
- * Returns sorted mailbox lists in several different ways. 
+/**
+ * Mailboxes with some chars (like -) can mess up the order, this fixes it
+ */
+function mailtree_sort(&$lsub) {
+    if(!is_array($lsub)) return;
+    
+    global $delimiter;
+    
+    foreach($lsub as $index => $mailbox)
+        $lsub[$index] = str_replace($delimiter,' -#- ',$lsub[$index]);
+
+    usort($lsub, 'user_strcasecmp');
+
+    foreach($lsub as $index => $mailbox)
+        $lsub[$index] = str_replace(' -#- ',$delimiter,$lsub[$index]);
+}
+
+/**
+ * Returns sorted mailbox lists in several different ways.
  * See comment on sqimap_mailbox_parse() for info about the returned array.
  */
+
+
 function sqimap_mailbox_list($imap_stream, $force=false) {
     global $default_folder_prefix;
 
@@ -518,15 +591,15 @@ function sqimap_mailbox_list($imap_stream, $force=false) {
         for ($i = 0, $cnt = count($lsub_ary);$i < $cnt; $i++) {
             /*
              * Workaround for mailboxes returned as literal
-             * Doesn't work if the mailbox name is multiple lines 
-	     * (larger then fgets buffer)
+             * Doesn't work if the mailbox name is multiple lines
+             * (larger then fgets buffer)
              */
             if (isset($lsub_ary[$i + 1]) && substr($lsub_ary[$i],-3) == "}\r\n") {
-		if (ereg("^(\\* [A-Z]+.*)\\{[0-9]+\\}([ \n\r\t]*)$",
+                if (ereg("^(\\* [A-Z]+.*)\\{[0-9]+\\}([ \n\r\t]*)$",
                      $lsub_ary[$i], $regs)) {
-            	    $i++;
-            	    $lsub_ary[$i] = $regs[1] . '"' . addslashes(trim($lsub_ary[$i])) . '"' . $regs[2];
-		}
+                        $i++;
+                        $lsub_ary[$i] = $regs[1] . '"' . addslashes(trim($lsub_ary[$i])) . '"' . $regs[2];
+                }
             }
             $temp_mailbox_name = find_mailbox_name($lsub_ary[$i]);
             $sorted_lsub_ary[] = $temp_mailbox_name;
@@ -534,20 +607,20 @@ function sqimap_mailbox_list($imap_stream, $force=false) {
                 $inbox_subscribed = true;
             }
         }
-	/* remove duplicates */
-	$sorted_lsub_ary = array_unique($sorted_lsub_ary);
-
-	/* natural sort mailboxes */
+        /* remove duplicates */
+        $sorted_lsub_ary = array_unique($sorted_lsub_ary);
+       
+        /* natural sort mailboxes */
         if (isset($sorted_lsub_ary)) {
-            usort($sorted_lsub_ary, 'user_strcasecmp');
+            mailtree_sort($sorted_lsub_ary);
         }
-	/*
-	 * The LSUB response doesn't provide us information about \Noselect
-	 * mail boxes. The LIST response does, that's why we need to do a LIST
-	 * call to retrieve the flags for the mailbox
+        /*
+         * The LSUB response doesn't provide us information about \Noselect
+         * mail boxes. The LIST response does, that's why we need to do a LIST
+         * call to retrieve the flags for the mailbox
            * Note: according RFC2060 an imap server may provide \NoSelect flags in the LSUB response.
            * in other words, we cannot rely on it.
-	 */
+         */
         $sorted_list_ary = array();
         for ($i=0; $i < count($sorted_lsub_ary); $i++) {
             if (substr($sorted_lsub_ary[$i], -1) == $delimiter) {
@@ -563,10 +636,10 @@ function sqimap_mailbox_list($imap_stream, $force=false) {
             /* Another workaround for literals */
 
             if (isset($read[1]) && substr($read[1],-3) == "}\r\n") {
-		if (ereg("^(\\* [A-Z]+.*)\\{[0-9]+\\}([ \n\r\t]*)$",
+                if (ereg("^(\\* [A-Z]+.*)\\{[0-9]+\\}([ \n\r\t]*)$",
                      $read[0], $regs)) {
-            	    $read[0] = $regs[1] . '"' . addslashes(trim($read[1])) . '"' . $regs[2];
-		}
+                    $read[0] = $regs[1] . '"' . addslashes(trim($read[1])) . '"' . $regs[2];
+                }
             }
 
             if (isset($read[0])) {
@@ -585,11 +658,11 @@ function sqimap_mailbox_list($imap_stream, $force=false) {
                                              true, $response, $message);
             /* Another workaround for literals */
             if (isset($inbox_ary[1]) && substr($inbox_ary[0],-3) == "}\r\n") {
-		if (ereg("^(\\* [A-Z]+.*)\\{[0-9]+\\}([ \n\r\t]*)$",
+                if (ereg("^(\\* [A-Z]+.*)\\{[0-9]+\\}([ \n\r\t]*)$",
                      $inbox_ary[0], $regs)) {
-            	    $inbox_ary[0] = $regs[1] . '"' . addslashes(trim($inbox_ary[1])) .
+                    $inbox_ary[0] = $regs[1] . '"' . addslashes(trim($inbox_ary[1])) .
                                 '"' . $regs[2];
-		}
+                }
             }
             $sorted_list_ary[] = $inbox_ary[0];
             $sorted_lsub_ary[] = find_mailbox_name($inbox_ary[0]);
@@ -602,14 +675,15 @@ function sqimap_mailbox_list($imap_stream, $force=false) {
 
         /* Find INBOX */
         $cnt = count($boxesall);
-	$used = array_pad($used,$cnt,false);
+        $used = array_pad($used,$cnt,false);
         for($k = 0; $k < $cnt; ++$k) {
             if (strtolower($boxesall[$k]['unformatted']) == 'inbox') {
                 $boxesnew[] = $boxesall[$k];
                 $used[$k] = true;
-		break;
+                break;
             }
         }
+
         /* List special folders and their subfolders, if requested. */
         if ($list_special_folders_first) {
             for($k = 0; $k < $cnt; ++$k) {
@@ -617,8 +691,19 @@ function sqimap_mailbox_list($imap_stream, $force=false) {
                     $boxesnew[] = $boxesall[$k];
                     $used[$k]   = true;
                 }
-	    }
-	}
+            }
+        }
+
+
+        /* Find INBOX's children */
+        for($k = 0; $k < $cnt; ++$k) {
+            if (!$used[$k] && isBoxBelow(strtolower($boxesall[$k]['unformatted']), 'inbox') &&
+                strtolower($boxesall[$k]['unformatted']) != 'inbox') {
+                $boxesnew[] = $boxesall[$k];
+                $used[$k] = true;
+            }
+        }
+
 
         /* Rest of the folders */
         for($k = 0; $k < $cnt; $k++) {
@@ -631,7 +716,7 @@ function sqimap_mailbox_list($imap_stream, $force=false) {
     return $boxesnew;
 }
 
-/*
+/**
  *  Returns a list of all folders, subscribed or not
  */
 function sqimap_mailbox_list_all($imap_stream) {
@@ -722,198 +807,3 @@ function sqimap_mailbox_list_all($imap_stream) {
     return $boxes;
 }
 
-function sqimap_mailbox_tree($imap_stream) {
-    global $boxesnew, $default_folder_prefix, $unseen_notify, $unseen_type;
-    if (!isset($boxesnew)) {
-
-        global $data_dir, $username, $list_special_folders_first,
-               $folder_prefix, $delimiter, $trash_folder, $move_to_trash;
-
-
-        $inbox_in_list = false;
-        $inbox_subscribed = false;
-
-        require_once(SM_PATH . 'include/load_prefs.php');
-
-        /* LSUB array */
-        $lsub_ary = sqimap_run_command ($imap_stream, "LSUB \"$folder_prefix\" \"*\"",
-                                        true, $response, $message);
-
-        /*
-         * Section about removing the last element was removed 
-         * We don't return "* OK" anymore from sqimap_read_data
-         */
-        $sorted_lsub_ary = array();
-        $cnt = count($lsub_ary);
-        for ($i = 0; $i < $cnt; $i++) {
-            /*
-             * Workaround for EIMS
-             * Doesn't work if the mailbox name is multiple lines
-             */
-            if (isset($lsub_ary[$i + 1]) &&
-                ereg("^(\\* [A-Z]+.*)\\{[0-9]+\\}([ \n\r\t]*)$",
-                     $lsub_ary[$i], $regs)) {
-                $i++;
-                $lsub_ary[$i] = $regs[1] . '"' . addslashes(trim($lsub_ary[$i])) . '"' . $regs[2];
-            }
-
-            /*
-            if (preg_match("/^\*\s+LSUB\s+\((.*)\)\s+\"(.*)\"\s+\"?(.+(?=\")|.+).*$/",$lsub_ary[$i],$regs)) {
-                $flag = $regs[1];
-                $mbx = trim($regs[3]);
-                $sorted_lsub_ary[] = array ('mbx' => $mbx, 'flag' => $flag); 
-            }
-            */
-            $mbx = find_mailbox_name($lsub_ary[$i]);
-            $noselect = check_is_noselect($lsub_ary[$i]);
-            if (substr($mbx, -1) == $delimiter) {
-                $mbx = substr($mbx, 0, strlen($mbx) - 1);
-            }
-            $sorted_lsub_ary[] = array ('mbx' => $mbx, 'noselect' => $noselect); 
-        }
-        array_multisort($sorted_lsub_ary, SORT_ASC, SORT_REGULAR);
-
-        for($i = 0; $i < $cnt; $i++) {
-            if ($sorted_lsub_ary[$i]['mbx'] == 'INBOX') {
-                $inbox_in_list = true;
-                break;
-            }
-        }
-
-        /*
-         * Just in case they're not subscribed to their inbox,
-         * we'll get it for them anyway
-         */
-        if (!$inbox_in_list) {
-            $inbox_ary = sqimap_run_command ($imap_stream, "LIST \"\" \"INBOX\"",
-                                             true, $response, $message);
-            /* Another workaround for EIMS */
-            if (isset($inbox_ary[1]) &&
-                ereg("^(\\* [A-Z]+.*)\\{[0-9]+\\}([ \n\r\t]*)$",
-                     $inbox_ary[0], $regs)) {
-                $inbox_ary[0] = $regs[1] . '"' . addslashes(trim($inbox_ary[1])) .
-                                '"' . $regs[2];
-            }
-            $mbx = find_mailbox_name($inbox_ary[0]);
-            if (substr($mbx, -1) == $delimiter) {
-                $mbx = substr($mbx, 0, strlen($mbx) - 1);
-            }
-            if ($mbx == 'INBOX') {
-                $sorted_lsub_ary[] = array ('mbx' => $mbx, 'flag' => ''); 
-                sqimap_subscribe($imap_stream, 'INBOX');
-                $cnt++;
-            }
-
-            /*
-            if (preg_match("/^\*\s+LIST\s+\((.*)\)\s+\"(.*)\"\s+\"?(.+(?=\")|.+).*$/",$inbox_ary[0],$regs)) {
-                $flag = $regs[1];
-                $mbx = trim($regs[3]);
-                if (substr($mbx, -1) == $delimiter) {
-                    $mbx = substr($mbx, 0, strlen($mbx) - 1);
-                }
-                $sorted_lsub_ary[] = array ('mbx' => $mbx, 'flag' => $flag); 
-            }
-            */
-        }
-        for ($i = 0 ; $i < $cnt; $i++) {
-            $mbx = $sorted_lsub_ary[$i]['mbx'];
-            if (($unseen_notify == 2 && $mbx == 'INBOX') ||
-                ($unseen_notify == 3) ||
-                ($move_to_trash && ($mbx == $trash_folder))) {
-                if($sorted_lsub_ary[$i]['noselect']) {
-                    $sorted_lsub_ary[$i]['unseen'] = 0;
-                } else {
-                    $sorted_lsub_ary[$i]['unseen'] = 
-                        sqimap_unseen_messages($imap_stream, $mbx);
-                }
-                if (($unseen_type == 2) ||
-                    ($move_to_trash && ($mbx == $trash_folder)) ||
-                    ($mbx == $trash_folder)) {
-                    if($sorted_lsub_ary[$i]['noselect']) {
-                        $sorted_lsub_ary[$i]['nummessages'] = 0;
-                    } else {
-                        $sorted_lsub_ary[$i]['nummessages'] =
-                            sqimap_get_num_messages($imap_stream, $mbx);
-                    }
-                }
-            }
-        }
-        $boxesnew = sqimap_fill_mailbox_tree($sorted_lsub_ary);
-        return $boxesnew;
-    }
-}
-
-
-function sqimap_fill_mailbox_tree($mbx_ary, $mbxs=false) {
-    global $data_dir, $username, $list_special_folders_first,
-           $folder_prefix, $trash_folder, $sent_folder, $draft_folder,
-           $move_to_trash, $move_to_sent, $save_as_draft,
-           $delimiter;
-
-    $special_folders = array ('INBOX', $sent_folder, $draft_folder, $trash_folder);
-
-    /* create virtual root node */
-    $mailboxes= new mailboxes();
-    $mailboxes->is_root = true;
-    $trail_del = false;
-    if (isset($folder_prefix) && $folder_prefix != '') {
-        $start = substr_count($folder_prefix,$delimiter);
-        if (strrpos($folder_prefix, $delimiter) == (strlen($folder_prefix)-1)) {
-            $trail_del = true;
-            $mailboxes->mailboxname_full = substr($folder_prefix,0, (strlen($folder_prefix)-1));
-        } else {
-            $mailboxes->mailboxname_full = $folder_prefix;
-            $start++;
-        }
-        $mailboxes->mailboxname_sub = $mailboxes->mailboxname_full;
-    } else {
-        $start = 0;
-    }
-    $cnt =  count($mbx_ary);
-    for ($i=0; $i < $cnt; $i++) {
-        if ($mbx_ary[$i]['mbx'] !='' ) {
-            $mbx = new mailboxes();
-            $mailbox = $mbx_ary[$i]['mbx'];
-            switch ($mailbox) {
-                case 'INBOX':
-                    $mbx->is_inbox = true;
-                    $mbx->is_special = true;
-                    break;
-                case $trash_folder:
-                    $mbx->is_trash = true;
-                    $mbx->is_special = true;
-                    break;
-                case $sent_folder:
-                    $mbx->is_sent = true;
-                    $mbx->is_special = true;
-                    break;
-                case $draft_folder:
-                    $mbx->is_draft = true;
-                    $mbx->is_special = true;
-                    break;
-            }
-
-            if (isset($mbx_ary[$i]['unseen'])) {
-                $mbx->unseen = $mbx_ary[$i]['unseen'];
-            }
-            if (isset($mbx_ary[$i]['nummessages'])) {
-                $mbx->total = $mbx_ary[$i]['nummessages'];
-            }
-
-            $mbx->is_noselect = $mbx_ary[$i]['noselect'];
-
-            $r_del_pos = strrpos($mbx_ary[$i]['mbx'], $delimiter);
-            if ($r_del_pos) {
-                $mbx->mailboxname_sub = substr($mbx_ary[$i]['mbx'],$r_del_pos+1);
-            } else {   /* mailbox is root folder */
-                $mbx->mailboxname_sub = $mbx_ary[$i]['mbx'];
-            }
-            $mbx->mailboxname_full = $mbx_ary[$i]['mbx'];
-            $mailboxes->addMbx($mbx, $delimiter, $start, $list_special_folders_first);
-        }
-    }
-
-    return $mailboxes;
-}
-
-?>
