@@ -5,9 +5,9 @@
  *
  * Displays all optinos about display preferences
  *
- * @copyright &copy; 1999-2007 The SquirrelMail Project Team
+ * @copyright &copy; 1999-2009 The SquirrelMail Project Team
  * @license http://opensource.org/licenses/gpl-license.php GNU Public License
- * @version $Id: display.php 12718 2007-10-01 19:37:52Z pdontthink $
+ * @version $Id: display.php 13549 2009-04-15 22:00:49Z jervfors $
  * @package squirrelmail
  */
 
@@ -118,7 +118,8 @@ function load_optpage_data_display() {
     );
 
     if ($optmode != 'submit')
-       $onLoadScript = 'document.forms[0].new_js_autodetect_results.value = \'' . SMPREF_JS_ON . '\'';
+       // FIXME: our form is named "f" - why? let's use something better
+       $onLoadScript = 'document.f.new_js_autodetect_results.value = \'' . SMPREF_JS_ON . '\'';
     else
        $onLoadScript = '';
 
@@ -435,4 +436,3 @@ function save_option_reply_prefix($option) {
 
 }
 
-?>
