@@ -5,14 +5,17 @@
  * This script creates separate page, that allows to review and modify
  * SquirrelMail configuration file.
  *
- * @version $Id: options.php,v 1.32.2.8 2005/03/23 15:20:02 tokul Exp $
+ * @version $Id: options.php 13549 2009-04-15 22:00:49Z jervfors $
  * @author Philippe Mingo
- * @copyright (c) 1999-2005 The SquirrelMail Project Team
+ * @copyright (c) 1999-2009 The SquirrelMail Project Team
  * @license http://opensource.org/licenses/gpl-license.php GNU Public License
  * @package plugins
  * @subpackage administrator
  */
 
+/** This is the administrator_options page */
+define('PAGE_NAME', 'administrator_options');
+
 /**
  * parse the config file
  *
@@ -164,6 +167,7 @@ function change_to_rel_path($old_path) {
  *     ../images/logo.gif      --> SM_PATH . 'images/logo.gif'
  *     images/logo.gif         --> SM_PATH . 'config/images/logo.gif'
  *     /absolute/path/logo.gif --> '/absolute/path/logo.gif'
+ *     C:/absolute/win/path    --> 'C:/absolute/win/path'
  *     http://whatever/        --> 'http://whatever'
  *  
  * @param string $old_path path that has to be converted
@@ -173,7 +177,8 @@ function change_to_rel_path($old_path) {
 function change_to_sm_path($old_path) {
     if ( $old_path === '' || $old_path == "''" ) {
         return "''";
-    } elseif ( preg_match("/^(\/|http)/", $old_path) ) {
+    } elseif ( preg_match("/^(\/|http)/", $old_path) ||
+        substr($old_path,1,2) == ':/' ) {
         return "'" . $old_path . "'";
     } elseif ( preg_match("/^(\$|SM_PATH)/", $old_path) ) {
         return $old_path;
@@ -631,4 +636,4 @@ if ( $fp = @fopen( $cfgfile, 'w' ) ) {
          '</big></p>';
 }
 ?>
-</body></html>
\ No newline at end of file
+</body></html>
