@@ -18,8 +18,8 @@
  *
  * Also view plugins/README.plugins for more information.
  *
- * @version $Id: filters.php,v 1.45.2.11 2004/12/28 10:49:35 tokul Exp $
- * @copyright (c) 1999-2005 The SquirrelMail Project Team
+ * @version $Id: filters.php 13724 2009-05-20 15:21:14Z kink $
+ * @copyright (c) 1999-2009 The SquirrelMail Project Team
  * @license http://opensource.org/licenses/gpl-license.php GNU Public License
  * @package plugins
  * @subpackage filters
@@ -185,6 +185,28 @@ function start_filters() {
     sqgetGlobalVar('username', $username, SQ_SESSION);
     sqgetGlobalVar('key',      $key,      SQ_COOKIE);
 
+
+    $filters = load_filters();
+
+    // No point running spam filters if there aren't any to run //
+    if ($AllowSpamFilters) {
+        $spamfilters = load_spam_filters();
+
+        $AllowSpamFilters = false;
+        foreach($spamfilters as $filterkey=>$value) {
+            if ($value['enabled'] == SMPREF_ON) {
+                $AllowSpamFilters = true;
+                break;
+            }
+        }
+    }
+
+    // No user filters, and no spam filters, no need to continue //
+    if (!$AllowSpamFilters && empty($filters)) {
+        return;
+    }
+
+
     // Detect if we have already connected to IMAP or not.
     // Also check if we are forced to use a separate IMAP connection
     if ((!isset($imap_stream) && !isset($imapConnection)) ||
@@ -265,12 +287,19 @@ function filter_search_and_delete($imap, $where, $what, $where_to, $user_scan,
                                   $del_id) {
     global $languages, $squirrelmail_language, $allow_charset_search, 
            $uid_support, $imap_server_type;
+
+    if (strtolower($where_to) == 'inbox') {
+        return array();
+    }
+
     if ($user_scan == 'new') {
         $category = 'UNSEEN';
     } else {
         $category = 'ALL';
     }
 
+    $category .= ' UNDELETED';
+
     if ($allow_charset_search &&
         isset($languages[$squirrelmail_language]['CHARSET']) &&
         $languages[$squirrelmail_language]['CHARSET']) {
@@ -291,7 +320,7 @@ function filter_search_and_delete($imap, $where, $what, $where_to, $user_scan,
         $search_str .= ' ' . $where . ' ' . $what;
     } else {
         $search_str .= ' ' . $where . ' {' . strlen($what) . "}\r\n"
-                    . $what . "\r\n";
+                    . $what;
     }
 
     /* read data back from IMAP */
@@ -304,11 +333,16 @@ function filter_search_and_delete($imap, $where, $what, $where_to, $user_scan,
     if ($response == 'OK') {
         $ids = explode(' ', $read[$r]);
         if (sqimap_mailbox_exists($imap, $where_to)) {
+            /*
+             * why we do n calls instead of just one. It is safer to copy 
+             * messages one by one, but code does not call expunge after 
+             * message is copied and quota limits are not resolved.
+             */
             for ($j=2, $num = count($ids); $j < $num; $j++) {
                 $id = trim($ids[$j]);
-                $del_id[] = $id;
-                sqimap_messages_copy ($imap, $id, $id, $where_to);
-                sqimap_messages_flag ($imap, $id, $id, 'Deleted',false);
+                if (sqimap_msgs_list_move($imap, $id, $where_to)) {
+                    $del_id[] = $id;
+                }
             }
         }
     }
@@ -334,116 +368,102 @@ function spam_filters($imap_stream) {
        filters_LoadCache();
     }
 
-    $run = 0;
+    $run = false;
 
     foreach ($filters as $Key=> $Value) {
         if ($Value['enabled']) {
-            $run ++;
+            $run = true;
+            break;
         }
     }
 
     // short-circuit
-    if ($run == 0) {
+    if (!$run) {
         return;
     }
 
     sqimap_mailbox_select($imap_stream, 'INBOX');
 
-    // Ask for a big list of all "Received" headers in the inbox with
-    // flags for each message.  Kinda big.
-    if ($filters_spam_scan != 'new') {
-        $read = sqimap_run_command($imap_stream, 'FETCH 1:* (FLAGS BODY.PEEK[HEADER.FIELDS ' .
-            '(RECEIVED)])', true, $response, $message, $uid_support);
-    } else {
+    $search_array = array();
+    if ($filters_spam_scan == 'new') {
         $read = sqimap_run_command($imap_stream, 'SEARCH UNSEEN', true, $response, $message, $uid_support);
-        if ($response != 'OK' || trim($read[0]) == '* SEARCH') {
-            $read = sqimap_run_command($imap_stream, 'FETCH 1:* (FLAGS BODY.PEEK[HEADER.FIELDS ' .
-            '(RECEIVED)])', true, $response, $message, $uid_support);
-        } else {
-            if (isset($read[0])) {
-                if (preg_match("/^\* SEARCH (.+)$/", $read[0], $regs)) {
-                    $search_array = preg_split("/ /", trim($regs[1]));
+        if (isset($read[0])) {
+            for ($i = 0, $iCnt = count($read); $i < $iCnt; ++$i) {
+                if (preg_match("/^\* SEARCH (.+)$/", $read[$i], $regs)) {
+                    $search_array = explode(' ', trim($regs[1]));
+                    break;
                 }
             }
-            $msgs_str = sqimap_message_list_squisher($search_array);
-            $imap_query = 'FETCH '.$msgs_str;
-            $imap_query .= ' (FLAGS BODY.PEEK[HEADER.FIELDS ';
-            $imap_query .= '(RECEIVED)])';
-            $read = sqimap_run_command($imap_stream,$imap_query, true, $response, $message, $uid_support);
         }
     }
     
+    if ($filters_spam_scan == 'new' && count($search_array)) {
+        $msg_str = sqimap_message_list_squisher($search_array);
+        $imap_query = 'FETCH ' . $msg_str . ' (FLAGS BODY.PEEK[HEADER.FIELDS (RECEIVED)])';
+    } else if ($filters_spam_scan != 'new') {
+        $imap_query = 'FETCH 1:* (FLAGS BODY.PEEK[HEADER.FIELDS (RECEIVED)])';
+    } else {
+        return;
+    }
+    
+    $read = sqimap_run_command_list($imap_stream, $imap_query, true, $response, $message, $uid_support);
+    
     if (isset($response) && $response != 'OK') {
         return;
     }
+    
+    $messages = parseFetch($read);
+    
+    $bulkquery = (strlen($SpamFilters_BulkQuery) > 0 ? true : false);
 
-    if (strlen($SpamFilters_BulkQuery) > 0) {
-       filters_bulkquery($filters_spam_scan, $filters, $read);
-    }
-
-    $i = 0;
-    while ($i < count($read)) {
-        // EIMS will give funky results
-        $Chunks = explode(' ', $read[$i]);
-        if ($Chunks[0] != '*') {
-            $i ++;
-            continue;
+    $aSpamIds = array();
+    foreach($messages as $id=>$message) {
+        if (isset($message['UID'])) {
+            $MsgNum = $message['UID'];
+        } else {
+            $MsgNum = $id;
         }
-        $MsgNum = $Chunks[1];
-
-        $IPs = array();
-        $i ++;
-        $IsSpam = 0;
-
-        // Look through all of the Received headers for IP addresses
-        // Stop when I get ")" on a line
-        // Stop if I get "*" on a line (don't advance)
-        // and above all, stop if $i is bigger than the total # of lines
-        while (($i < count($read)) &&
-                ($read[$i][0] != ')' && $read[$i][0] != '*' &&
-                $read[$i][0] != "\n") && (! $IsSpam)) {
-            // Check to see if this line is the right "Received from" line
-            // to check
-            if (is_int(strpos($read[$i], $SpamFilters_YourHop))) {
-
-                // short-circuit and skip work if we don't scan this one
-                $read[$i] = ereg_replace('[^0-9\.]', ' ', $read[$i]);
-                $elements = explode(' ', $read[$i]);
-                foreach ($elements as $value) {
-                    if ($value != '' &&
-                        ereg('[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}',
-                            $value, $regs)) {
-                        $Chunks = explode('.', $value);
-                        if (filters_spam_check_site($Chunks[0],
-                                $Chunks[1], $Chunks[2], $Chunks[3],
-                                $filters)) {
-                            $IsSpam ++;
-                            break;  // no sense in checking more IPs
+        
+        if (isset($message['received'])) {
+            foreach($message['received'] as $received) {
+                if (is_int(strpos($received, $SpamFilters_YourHop))) {
+                    if (preg_match('/([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})/', $received, $matches)) {
+                        $IsSpam = false;
+                        if (filters_spam_check_site($matches[1], $matches[2], $matches[3], $matches[4], $filters)) {
+                            $aSpamIds[] = $MsgNum;
+                            $IsSpam = true;                            
                         }
-                        // If we've checked one IP and YourHop is
-                        // just a space
-                        if ($SpamFilters_YourHop == ' ') {
-                            break;  // don't check any more
+                        
+                        if ($bulkquery) {
+                            array_shift($matches);
+                            $IP = explode('.', $matches);
+                            foreach($filters as $key=>$value) {
+                                if ($filters[$key]['enabled'] && $filters[$key]['dns']) {
+                                    if (strlen($SpamFilters_DNScache[$IP . '.' . $filters[$key]['dns']]) == 0) {
+                                        $IPs[$IP] = true;
+                                        break;
+                                    }
+                                }
+                            }
+                        }
+                        
+                        if ($SpamFilters_YourHop == ' ' || $IsSpam) {
+                            break;
                         }
                     }
                 }
             }
-            $i ++;
-        }
-
-        // Lookie!  It's spam!  Yum!
-        if ($IsSpam) {
-            if (sqimap_mailbox_exists($imap_stream, $filters_spam_folder)) {
-                sqimap_messages_copy ($imap_stream, $MsgNum, $MsgNum,
-                                    $filters_spam_folder);
-                sqimap_messages_flag ($imap_stream, $MsgNum, $MsgNum,
-                                    'Deleted', false);
-            }
-        } else {
         }
     }
-
-    sqimap_mailbox_expunge($imap_stream, 'INBOX');
+    
+    if (count($aSpamIds) && sqimap_mailbox_exists($imap_stream, $filters_spam_folder)) {
+        sqimap_msgs_list_move($imap_stream, $aSpamIds, $filters_spam_folder);
+        sqimap_mailbox_expunge($imap_stream, 'INBOX', true, $aSpamIds);
+    }
+    
+    if ($bulkquery && count($IPs)) {
+        filters_bulkquery($filters, $IPs);
+    }
 
     if ($SpamFilters_SharedCache) {
        filters_SaveCache();
@@ -464,8 +484,13 @@ function filters_spam_check_site($a, $b, $c, $d, &$filters) {
     foreach ($filters as $key => $value) {
         if ($filters[$key]['enabled']) {
             if ($filters[$key]['dns']) {
+                
+                /**
+                 *  RFC allows . on end of hostname to ensure search domain
+                 *  isn't used if no hostname is found
+                 */
                 $filter_revip = $d . '.' . $c . '.' . $b . '.' . $a . '.' .
-                                $filters[$key]['dns'];
+                                $filters[$key]['dns'] . '.';
 
                 if(!isset($SpamFilters_DNScache[$filter_revip]['L']))
                         $SpamFilters_DNScache[$filter_revip]['L'] = '';
@@ -479,8 +504,12 @@ function filters_spam_check_site($a, $b, $c, $d, &$filters) {
                     $SpamFilters_DNScache[$filter_revip]['T'] =
                                        time() + $SpamFilters_CacheTTL;
                 }
-                if ($SpamFilters_DNScache[$filter_revip]['L'] ==
-                    $filters[$key]['result']) {
+                
+                /**
+                 *  gethostbyname returns ip if resolved, or returns original host
+                 *  supplied to function if there is no resolution
+                 */
+                if ($SpamFilters_DNScache[$filter_revip]['L'] != $filter_revip) {
                     return 1;
                 }
             }
@@ -500,7 +529,7 @@ function load_filters() {
     for ($i=0; $fltr = getPref($data_dir, $username, 'filter' . $i); $i++) {
         $ary = explode(',', $fltr);
         $filters[$i]['where'] = $ary[0];
-        $filters[$i]['what'] = $ary[1];
+        $filters[$i]['what'] = str_replace('###COMMA###', ',', $ary[1]);
         $filters[$i]['folder'] = $ary[2];
     }
     return $filters;
@@ -563,14 +592,6 @@ function load_spam_filters() {
             _("COMMERCIAL - RBL+ Dial-up entries.");
     }
 
-    $filters['ORDB']['prefname'] = 'filters_spam_ordb';
-    $filters['ORDB']['name'] = 'Open Relay Database List';
-    $filters['ORDB']['link'] = 'http://www.ordb.org/';
-    $filters['ORDB']['dns'] = 'relays.ordb.org';
-    $filters['ORDB']['result'] = '127.0.0.2';
-    $filters['ORDB']['comment'] =
-        _("FREE - ORDB was born when ORBS went off the air. It seems to have fewer false positives than ORBS did though.");
-
     $filters['FiveTen Direct']['prefname'] = 'filters_spam_fiveten_src';
     $filters['FiveTen Direct']['name'] = 'Five-Ten-sg.com Direct SPAM Sources';
     $filters['FiveTen Direct']['link'] = 'http://www.five-ten-sg.com/blackhole.php';
@@ -780,8 +801,7 @@ function load_spam_filters() {
         _("FREE - Distributed Sender Boycott List - UN-Confirmed Relays");
 
     foreach ($filters as $Key => $Value) {
-        $filters[$Key]['enabled'] = getPref($data_dir, $username,
-            $filters[$Key]['prefname']);
+        $filters[$Key]['enabled'] = (bool)getPref($data_dir, $username, $filters[$Key]['prefname']);
     }
 
     return $filters;
@@ -863,4 +883,4 @@ function do_error($string) {
     echo $string;
     echo "</font></p>\n";
 }
-?>
\ No newline at end of file
+?>
