@@ -18,8 +18,8 @@
  *
  * Also view plugins/README.plugins for more information.
  *
- * @version $Id: options.php,v 1.26.2.11 2006/02/03 22:27:52 jervfors Exp $
- * @copyright (c) 1999-2006 The SquirrelMail Project Team
+ * @version $Id: options.php 13720 2009-05-19 01:49:24Z pdontthink $
+ * @copyright (c) 1999-2009 The SquirrelMail Project Team
  * @license http://opensource.org/licenses/gpl-license.php GNU Public License
  * @package plugins
  * @subpackage filters
@@ -79,7 +79,7 @@ if (sqgetGlobalVar('filter_submit',$filter_submit,SQ_POST)) {
     }
 
     if ($complete_post) {
-        $filter_what = str_replace(',', ' ', $filter_what);
+        $filter_what = str_replace(',', '###COMMA###', $filter_what);
         $filter_what = str_replace("\\\\", "\\", $filter_what);
         $filter_what = str_replace("\\\"", '"', $filter_what);
         $filter_what = str_replace('"', '&quot;', $filter_what);
@@ -265,7 +265,7 @@ if (sqgetGlobalVar('filter_submit',$filter_submit,SQ_POST)) {
         printf( _("If %s contains %s then move to %s"),
             '<b>'.$filters[$i]['where'].'</b>',
             '<b>'.$filters[$i]['what'].'</b>',
-            '<b>'.imap_utf7_decode_local($fdr).'</b>');
+            '<b>'.htmlspecialchars(imap_utf7_decode_local($fdr)).'</b>');
         echo '</td></tr>';
 
     }
@@ -276,4 +276,4 @@ if (sqgetGlobalVar('filter_submit',$filter_submit,SQ_POST)) {
             ) ,
         'center', '', 'width="80%" border="0" cellpadding="2" cellspacing="0"' );
     echo '</body></html>';
-?>
\ No newline at end of file
+?>
