@@ -1,15 +1,21 @@
 <?php
-/** Message Source  
-*
-* Plugin to view the RFC822 raw message output and the bodystructure of a message
-*
-* Copyright (c) 2002 Marc Groot Koerkamp, The Netherlands
-* Licensed under the GNU GPL. For full terms see the file COPYING.
-* 
-* $Id: setup.php,v 1.2.2.1 2004/04/25 09:19:01 tokul Exp $
-**/
-
+/**
+ * Message Details plugin - main setup script
+ *
+ * Plugin to view the RFC822 raw message output and the bodystructure of a message
+ *
+ * @author Marc Groot Koerkamp
+ * @copyright Copyright &copy; 2002 Marc Groot Koerkamp, The Netherlands
+ * @copyright Copyright &copy; 2004-2009 The SquirrelMail Project Team
+ * @license http://opensource.org/licenses/gpl-license.php GNU Public License
+ * @version $Id: setup.php 13549 2009-04-15 22:00:49Z jervfors $
+ * @package plugins
+ * @subpackage message_details
+ **/
 
+/**
+ * Initialize the plugin
+ */
 function squirrelmail_plugin_init_message_details()
 {
   global $squirrelmail_plugin_hooks;
@@ -18,14 +24,14 @@ function squirrelmail_plugin_init_message_details()
 }
 
 function show_message_details() {
-    global $passed_id, $mailbox, $ent_num, $color,
+    global $passed_id, $mailbox, $passed_ent_id, $color,
            $javascript_on;
 
     if (strlen(trim($mailbox)) < 1) {
         $mailbox = 'INBOX';
     }
 
-    $params = '?passed_ent_id=' . $ent_num .
+    $params = '?passed_ent_id=' . $passed_ent_id .
               '&mailbox=' . urlencode($mailbox) .
               '&passed_id=' . $passed_id;
 
