@@ -1,36 +1,29 @@
 <?php
-   /** 
-    **  setup.php -- SpamCop plugin           
-    **
-    **  Copyright (c) 1999-2005 The SquirrelMail Project Team
-    **  Licensed under the GNU GPL. For full terms see the file COPYING.
-    **  
-    **  $Id: setup.php,v 1.10.2.9 2005/06/15 21:12:07 kink Exp $                                                         
-    **/
-
-require_once(SM_PATH . 'functions/global.php');
+/** 
+ *  setup.php -- SpamCop plugin           
+ *
+ *  Copyright (c) 1999-2009 The SquirrelMail Project Team
+ *  Licensed under the GNU GPL. For full terms see the file COPYING.
+ *  
+ *  $Id: setup.php 13549 2009-04-15 22:00:49Z jervfors $                                                         
+ */
 
 /** Disable Quick Reporting by default */
+global $spamcop_quick_report;
 $spamcop_quick_report = false;
 
 /* Initialize the plugin */
 function squirrelmail_plugin_init_spamcop() {
-   global $squirrelmail_plugin_hooks, $data_dir, $username,
-      $spamcop_is_composing;
+   global $squirrelmail_plugin_hooks;
 
    $squirrelmail_plugin_hooks['optpage_register_block']['spamcop'] =
-      'spamcop_options';
+       'spamcop_options';
    $squirrelmail_plugin_hooks['loading_prefs']['spamcop'] =
-      'spamcop_load';
+       'spamcop_load';
    $squirrelmail_plugin_hooks['read_body_header_right']['spamcop'] =
-      'spamcop_show_link';
-
-    sqgetGlobalVar('spamcop_is_composing' , $spamcop_is_composing);
-      
-   if (isset($spamcop_is_composing)) {
-      $squirrelmail_plugin_hooks['compose_send']['spamcop'] =
-         'spamcop_while_sending';
-   }
+       'spamcop_show_link';
+   $squirrelmail_plugin_hooks['compose_send']['spamcop'] =
+       'spamcop_while_sending';
 }
 
 
@@ -51,12 +44,12 @@ function spamcop_load() {
 //      else
 
 // Default to web_form. It is faster.
-	$spamcop_method = 'web_form';
-	setPref($data_dir, $username, 'spamcop_method', $spamcop_method);
+        $spamcop_method = 'web_form';
+        setPref($data_dir, $username, 'spamcop_method', $spamcop_method);
     }
    if (! $spamcop_quick_report && $spamcop_method=='quick_email') {
-	$spamcop_method = 'web_form';
-	setPref($data_dir, $username, 'spamcop_method', $spamcop_method);
+       $spamcop_method = 'web_form';
+       setPref($data_dir, $username, 'spamcop_method', $spamcop_method);
    }
    if ($spamcop_id == '')
       $spamcop_enabled = 0;
@@ -92,29 +85,28 @@ function spamcop_show_link() {
        quick_email format - they will be updated after clicking the link
      */
     if (! $spamcop_quick_report && $spamcop_method=='quick_email') {
-	$spamcop_method = 'web_form';
+        $spamcop_method = 'web_form';
     }
    
    if ($spamcop_method == 'web_form') {
 ?><script language="javascript" type="text/javascript">
-document.write('<a href="../plugins/spamcop/spamcop.php?passed_id=<?PHP echo urlencode($passed_id); ?>&amp;js_web=1&amp;mailbox=<?PHP echo urlencode($mailbox); ?>&amp;passed_ent_id=<?PHP echo urlencode($passed_ent_id); ?>" target="_blank">');
-document.write("<?PHP echo _("Report as Spam"); ?>");
+document.write('<a href="../plugins/spamcop/spamcop.php?passed_id=<?php echo urlencode($passed_id); ?>&amp;js_web=1&amp;mailbox=<?php echo urlencode($mailbox); ?>&amp;passed_ent_id=<?php echo urlencode($passed_ent_id); ?>" target="_blank">');
+document.write("<?php echo _("Report as Spam"); ?>");
 document.write("</a>");
 </script><noscript>
-<a href="../plugins/spamcop/spamcop.php?passed_id=<?PHP echo urlencode($passed_id); ?>&amp;mailbox=<?PHP echo urlencode($mailbox); ?>&amp;startMessage=<?PHP echo urlencode($startMessage); ?>&amp;passed_ent_id=<?PHP echo urlencode($passed_ent_id); ?>">
-<?PHP echo _("Report as Spam"); ?></a>
-</noscript><?PHP
+<a href="../plugins/spamcop/spamcop.php?passed_id=<?php echo urlencode($passed_id); ?>&amp;mailbox=<?php echo urlencode($mailbox); ?>&amp;startMessage=<?php echo urlencode($startMessage); ?>&amp;passed_ent_id=<?php echo urlencode($passed_ent_id); ?>">
+<?php echo _("Report as Spam"); ?></a>
+</noscript><?php
    } else {
-?><a href="../plugins/spamcop/spamcop.php?passed_id=<?PHP echo urlencode($passed_id); ?>&amp;mailbox=<?PHP echo urlencode($mailbox); ?>&amp;startMessage=<?PHP echo urlencode($startMessage); ?>&amp;passed_ent_id=<?PHP echo urlencode($passed_ent_id); ?>">
-<?PHP echo _("Report as Spam"); ?></a>
-<?PHP
+?><a href="../plugins/spamcop/spamcop.php?passed_id=<?php echo urlencode($passed_id); ?>&amp;mailbox=<?php echo urlencode($mailbox); ?>&amp;startMessage=<?php echo urlencode($startMessage); ?>&amp;passed_ent_id=<?php echo urlencode($passed_ent_id); ?>">
+<?php echo _("Report as Spam"); ?></a>
+<?php
    }
 }
 
 
 // Show the link to our own custom options page
-function spamcop_options()
-{
+function spamcop_options() {
    global $optpage_blocks;
    
    $optpage_blocks[] = array(
@@ -127,20 +119,25 @@ function spamcop_options()
 
 
 // When we send the email, we optionally trash it then too
-function spamcop_while_sending()
-{
-   global $mailbox, $spamcop_delete, $spamcop_is_composing, $auto_expunge, 
+function spamcop_while_sending() {
+   global $mailbox, $spamcop_delete, $auto_expunge, 
       $username, $key, $imapServerAddress, $imapPort;
 
-   if ($spamcop_delete) {
-      $imapConnection = sqimap_login($username, $key, $imapServerAddress, 
-         $imapPort, 0);
-      sqimap_mailbox_select($imapConnection, $mailbox);
-      sqimap_messages_delete($imapConnection, $spamcop_is_composing, 
-         $spamcop_is_composing, $mailbox);
-      if ($auto_expunge)
-         sqimap_mailbox_expunge($imapConnection, $mailbox, true);
+   // load sqgetGlobalVar()
+   include_once(SM_PATH . 'functions/global.php');
+
+   // check if compose.php is called by spamcop plugin
+   if (sqgetGlobalVar('spamcop_is_composing' , $spamcop_is_composing)) {
+       if ($spamcop_delete) {
+           $imapConnection = sqimap_login($username, $key, $imapServerAddress, $imapPort, 0);
+           sqimap_mailbox_select($imapConnection, $mailbox);
+           sqimap_msgs_list_delete($imapConnection, $mailbox, $spamcop_is_composing);
+           if ($auto_expunge)
+               sqimap_mailbox_expunge($imapConnection, $mailbox, true);
+       }
+       // change default email composition setting. Plugin always operates in right frame.
+       // make sure that compose.php redirects to right page. Temporally override.
+       global $compose_new_win;
+       $compose_new_win = false;
    }
 }
-
-?>
