@@ -3,9 +3,9 @@
 /**
  * SquirrelMail configtest script
  *
- * @copyright &copy; 2003-2007 The SquirrelMail Project Team
+ * @copyright &copy; 2003-2009 The SquirrelMail Project Team
  * @license http://opensource.org/licenses/gpl-license.php GNU Public License
- * @version $Id: configtest.php 13238 2008-07-19 07:31:43Z pdontthink $
+ * @version $Id: configtest.php 13653 2009-05-05 23:09:28Z pdontthink $
  * @package squirrelmail
  * @subpackage config
  */
@@ -92,6 +92,16 @@ echo $IND . 'display_errors: ' . ini_get('display_errors') . "<br />\n";
 
 echo $IND . 'error_reporting: ' . ini_get('error_reporting') . "<br />\n";
 
+// FIXME: words like "probably" and "usually" should be removed and do_err() should be called with TRUE for the second argument when we confirm that this does in fact affect everyone
+if ((bool) ini_get('session.auto_start') && ini_get('session.auto_start') != 'off') {
+    $msg = 'session.auto_start is turned on in your PHP configuration, but SquirrelMail'
+         . ' 1.4.x probably will not work with it (otherwise valid logins will usually'
+         . ' result in "You must be logged in to access this page" errors).'
+         . ' You can disable session.auto_start only in the squirrelmail directory' 
+         . ' if you need to leave it turned on for other applications.';
+    do_err($msg, false);
+}
+
 $safe_mode = ini_get('safe_mode');
 if ($safe_mode) {
     echo $IND . 'safe_mode: ' . $safe_mode;
