@@ -6,12 +6,15 @@
  * top frame of printer_friendly_main.php
  * displays some javascript buttons for printing & closing
  *
- * @copyright &copy; 1999-2007 The SquirrelMail Project Team
+ * @copyright &copy; 1999-2009 The SquirrelMail Project Team
  * @license http://opensource.org/licenses/gpl-license.php GNU Public License
- * @version $Id: printer_friendly_top.php 12127 2007-01-13 20:07:24Z kink $
+ * @version $Id: printer_friendly_top.php 13549 2009-04-15 22:00:49Z jervfors $
  * @package squirrelmail
  */
 
+/** This is the printer_friendly_top page */
+define('PAGE_NAME', 'printer_friendly_top');
+
 /**
  * Path for SquirrelMail required files.
  * @ignore
